"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskArgv = exports.defaultHelpMsg = exports.loadConfigPromises = exports.defaultCli = void 0;
exports.outputCompletion = outputCompletion;
const tslib_1 = require("tslib");
const cac_1 = tslib_1.__importDefault(require("cac"));
const fs_1 = require("./fs");
const path_1 = tslib_1.__importDefault(require("path"));
const logger_1 = require("./logger");
const utils_1 = require("./utils");
exports.defaultCli = (0, cac_1.default)();
// generate default argv because cac cannot handle `foy command` + `foy options`
function generateArgv() {
    let defaultArgv = [];
    let taskArgv = [];
    const argv = process.argv.slice(2);
    const defaultOptions = new Map([
        ['--config', 1],
        ['-c', 1],
        ['--require', 1],
        ['-r', 1],
        ['--init', 1],
        ['-i', 1],
        ['--completion', 1],
        ['--completion-profile', 0],
    ]);
    let i = 0;
    for (i = 0; i < argv.length; i++) {
        const arg = argv[i];
        let valLen = defaultOptions.get(arg);
        if (utils_1.Is.defed(valLen)) {
            let end = i + valLen;
            defaultArgv.push(...argv.slice(i, end + 1));
            i = end;
        }
        else {
            break;
        }
    }
    taskArgv = argv.slice(i);
    defaultArgv = process.argv.slice(0, 2).concat(defaultArgv);
    taskArgv = process.argv.slice(0, 2).concat(taskArgv);
    return [defaultArgv, taskArgv];
}
const [defaultArgv, taskArgv] = generateArgv();
exports.taskArgv = taskArgv;
// parse default options
const defaultOptions = [
    [`--config, -c <...files>`, 'The Foyfiles'],
    [`--require, -r <...names>`, 'Require the given modules'],
    [`--init, -i [ext]`, 'Generate the Foyfile, [ext] can be "ts" | "js", default is "js"'],
    [`--completion [val]`, `Generate completion words`],
    [`--completion-profile`, `Generate completion shell profile`],
];
defaultOptions.forEach(([name, desc]) => {
    exports.defaultCli.option(name, desc);
});
exports.defaultCli.parse(defaultArgv);
if (exports.defaultCli.options.init) {
    let ext = exports.defaultCli.options.init;
    if (!utils_1.Is.str(ext)) {
        ext = 'js';
    }
    ext = ext.replace(/^\./, '');
    const file = `./Foyfile.${ext}`;
    if (fs_1.fs.existsSync(file)) {
        throw new Error(`Foyfile already exists: ${path_1.default.resolve(file)}`);
    }
    fs_1.fs.writeFileSync(file, `${ext === 'js'
        ? `const { task, desc, option, fs } = require('foy')`
        : `import { task, desc, option, fs } from 'foy'`}

  task('build', async ctx => {
    // Your build tasks
    await ctx.exec('tsc')
  })

  `);
    process.exit();
}
else if (exports.defaultCli.options.completionProfile) {
    console.log(`
###-begin-foy-completions-###
#
# foy command completion script
#
# Installation: foy --completion-profile >> ~/.bashrc
#    or foy --completion-profile >> ~/.zshrc

complete -F _foy_complete_func foy

_foy_complete_func()
{
    local cur opts
    COMPREPLY=()
    cur="\${COMP_WORDS[COMP_CWORD]}"
    opts="$(node ./lib/cli.ts --completion "\${COMP_WORDS[COMP_CWORD-1]}")"

    if [[ \${cur} == * ]] ; then
        COMPREPLY=( $(compgen -W "\${opts}" -- \${cur}) )
        return 0
    fi
}
###-end-foy-completions-###
  `);
    process.exit();
}
function arrify(arr) {
    if (!Array.isArray(arr)) {
        return arr == null ? [] : [arr];
    }
    return arr;
}
let foyFiles = arrify(exports.defaultCli.options.config);
let registers = arrify(exports.defaultCli.options.require);
if (foyFiles.length) {
    foyFiles = foyFiles.map((c) => path_1.default.resolve(process.cwd(), c));
}
else {
    // find default foyfiles
    let findFoyfiles = (baseDir) => {
        let cwdFoyfiles = fs_1.fs.readdirSync(baseDir).filter((f) => f.startsWith('Foyfile.'));
        if (cwdFoyfiles.length) {
            if (cwdFoyfiles.length > 1) {
                logger_1.logger.warn(`Find more than 1 Foyfile in current directory, only first one will be used: \n${cwdFoyfiles.join('\n')}`);
            }
            foyFiles = [path_1.default.join(baseDir, cwdFoyfiles[0])];
        }
    };
    findFoyfiles(process.cwd());
    if (!foyFiles.length) {
        let maxDepth = 5;
        let dir = process.cwd();
        while (maxDepth-- && dir !== '/' && dir && !foyFiles.length) {
            dir = path_1.default.dirname(dir);
            findFoyfiles(dir);
        }
    }
}
// check custom foyfiles exists
for (const file of foyFiles) {
    if (!fs_1.fs.existsSync(file)) {
        throw new TypeError(`Cannot find Foyfile: ${file}`);
    }
}
// add custom registers
if (registers.length) {
    for (let mod of registers) {
        try {
            require(mod);
        }
        catch (error) {
            require(path_1.default.resolve(process.cwd(), mod));
        }
    }
}
// add ts-node registry for ts foyfile
function isESM() {
    try {
        let pkg = fs_1.fs.readJsonSync('./package.json');
        return pkg.type === 'module';
    }
    catch (e) {
        return false;
    }
}
try {
    if (foyFiles.some((f) => f.endsWith('.ts')) && !require.extensions['.ts']) {
        let options = {
            transpileOnly: true,
            compilerOptions: {
                module: 'commonjs',
            },
        };
        require('ts-node').register(options);
    }
}
catch (error) {
    // ignore
}
{
    // Add global installed foy to module.paths if using global foy
    const Module = require('module');
    const nodeModulePaths = Module._nodeModulePaths;
    const globalFoyPath = path_1.default.join(__dirname, '..', '..');
    if (nodeModulePaths) {
        Module._nodeModulePaths = (...args) => {
            let paths = nodeModulePaths.apply(Module, args);
            if (Array.isArray(paths)) {
                paths.push(globalFoyPath);
            }
            return paths;
        };
    }
}
exports.loadConfigPromises = [];
// load foyfiles
for (const file of foyFiles) {
    if (isESM()) {
        exports.loadConfigPromises.push(Promise.resolve(`${file}`).then(s => tslib_1.__importStar(require(s))));
    }
    else {
        require(file);
    }
}
exports.defaultHelpMsg = defaultOptions
    .map(([name, desc]) => `  ${name}:${Array(30 - name.length)
    .fill(' ')
    .join('')}${desc}`)
    .join('\n');
function outputCompletion(taskCli) {
    let completVal = exports.defaultCli.options.completion;
    if (!completVal)
        return;
    if (completVal === 'foy') {
        completVal = '';
    }
    function getOptionComplets(cmd) {
        var _a, _b;
        return (_b = (_a = taskCli.commands
            .find((c) => c.name === cmd)) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.map((o) => '--' + o.name).join(' ');
    }
    const cmdComplets = taskCli.commands.map((c) => c.name);
    // .concat(defaultOptions.map((d) => d[0].split(/[,\s]/)[0]).filter(v => !v.includes('completion')))
    if (completVal && cmdComplets.includes(completVal)) {
        console.log(getOptionComplets(completVal));
    }
    else {
        console.log(cmdComplets.join(' '));
    }
    process.exit();
}
//# sourceMappingURL=default-cli.js.map